@@ -11,7 +11,7 @@ std::array<int, 3> gemm_algos = std::array<int, 3>({99, 99, 99});
 
 template <typename T>
 at::Tensor ds_softmax(at::Tensor& attn_scores,
-                      at::Tensor& attn_mask,
+                      T* attn_mask_ptr,
                       bool triangular,
                       bool recompute,
                       bool local_attention,
@@ -24,7 +24,7 @@ at::Tensor ds_softmax(at::Tensor& attn_scores,
     int heads = attn_scores_c.size(1);
 
     launch_attn_softmax_v2((T*)attn_scores_c.data_ptr(),
-                           (T*)attn_mask.data_ptr(),
+                           attn_mask_ptr,
                            triangular,
                            recompute,
                            local_attention,
@@ -42,7 +42,7 @@ at::Tensor ds_softmax(at::Tensor& attn_scores,
 template <typename T>
 void attention_unfused(at::Tensor& prev_key_cont,
                        at::Tensor& query_cont,
-                       at::Tensor& attn_mask,
+                       T* attn_mask_ptr,
                        at::Tensor& prev_value_cont,
                        at::Tensor& output,
                        int& bsz,
@@ -81,8 +81,8 @@ void attention_unfused(at::Tensor& prev_key_cont,
                                 seq_len * soft_len,
                                 bsz * heads,
                                 CUBLAS_GEMM_DEFAULT_TENSOR_OP);
-    attn_score =
-        ds_softmax<T>(attn_score, attn_mask, triangular, recompute, local_attention, window_size);
+    attn_score = ds_softmax<T>(
+        attn_score, attn_mask_ptr, triangular, recompute, local_attention, window_size);
     alpha = 1.0;
     cublas_strided_batched_gemm(Context::Instance().GetCublasHandle(),
                                 k,
@@ -114,7 +114,8 @@ std::vector<at::Tensor> ds_softmax_context(at::Tensor& query,
                                            bool merging,
                                            bool triangular,
                                            bool local_attention,
-                                           int window_size)
+                                           int window_size,
+                                           bool no_masking)
 {
     auto query_cont = query.contiguous();
     auto prev_key_cont = prev_key.contiguous();
@@ -138,7 +139,7 @@ std::vector<at::Tensor> ds_softmax_context(at::Tensor& query,
         at::empty({prev_value.size(0), heads, seq_len, prev_value.size(2) / heads}, options);
     attention_unfused<T>(prev_key_cont,
                          query_cont,
-                         attn_mask,
+                         (no_masking ? nullptr : (T*)attn_mask.data_ptr()),
                          prev_value_cont,
                          output,
                          bsz,
