@@ -16,24 +16,19 @@ import typing
 
 
 class MoE(torch.nn.Module):
-    '''
-        DeepSpeed MOE API: This defines a simple API that can be used from client-side code.
-    '''
     def __init__(self,
                  hidden_size,
                  expert,
                  num_experts=1,
                  k=1,
-                 output_dropout_prob=0.0,
                  capacity_factor=1.,
                  eval_capacity_factor=1.,
                  min_capacity=4,
                  noisy_gate_policy: typing.Optional[str] = None):
         """Initialize an MoE layer.
-        TODO: add details about input/output dimension assumptions
 
         Arguments:
-            hidden_size (int): the hidden dimension of the model.
+            hidden_size (int): the hidden dimension of the model, importantly this is also the input and output dimension.
 
             expert (torch.nn.Module): the torch module that defines the expert (e.g., MLP, torch.linear).
 
@@ -41,8 +36,6 @@ class MoE(torch.nn.Module):
 
             k (int, optional): default=1, top-k gating value, only supports k=1 or k=2.
 
-            output_dropout_prob (float, optional): default=0.0, output dropout probability.
-
             capacity_factor (float, optional): default=1.0, the capacity of the expert at training time.
 
             eval_capacity_factor (float, optional): default=1.0, the capacity of the expert at eval time.
@@ -78,19 +71,21 @@ class MoE(torch.nn.Module):
                                       num_local_experts,
                                       group=groups.get_expert_parallel_group())
 
-        self.dropout = torch.nn.Dropout(output_dropout_prob)
-
     def forward(self, hidden_states, used_token=None):
-        """
+        """ MoE forward
+
         Arguments:
             hidden_states (Tensor): input to the layer
             used_token (Tensor, optional): default: None, mask only used tokens
 
         Returns:
-            output (Tensor): output of the model
-            l_aux (Tensor): gate loss value
-            exp_counts (int): expert count
+            A tuple including output, gate loss, and expert count.
+
+            * output (Tensor): output of the model
+
+            * l_aux (Tensor): gate loss value
+
+            * exp_counts (int): expert count
         """
         output = self.deepspeed_moe(hidden_states, used_token)
-        output = self.dropout(output)
         return output, self.deepspeed_moe.l_aux, self.deepspeed_moe.exp_counts
