@@ -48,7 +48,7 @@ def installed_cuda_version():
     return int(cuda_major), int(cuda_minor)
 
 
-def get_default_compute_capatabilities():
+def get_default_compute_capabilities():
     compute_caps = DEFAULT_COMPUTE_CAPABILITIES
     import torch.utils.cpp_extension
     if torch.utils.cpp_extension.CUDA_HOME is not None and installed_cuda_version(
@@ -70,7 +70,8 @@ cuda_minor_mismatch_ok = {
     11: ["11.0",
          "11.1",
          "11.2",
-         "11.3"],
+         "11.3",
+         "11.4"],
 }
 
 
@@ -171,7 +172,7 @@ class OpBuilder(ABC):
         '''
         Test for existence of a function within a tuple of libraries.
 
-        This is used as a smoke test to check whether a certain library is avaiable.
+        This is used as a smoke test to check whether a certain library is available.
         As a test, this creates a simple C program that calls the specified function,
         and then distutils is used to compile that program and link it with the specified libraries.
         Returns True if both the compile and link are successful, False otherwise.
@@ -275,8 +276,15 @@ class OpBuilder(ABC):
                 "your system, will fall back to non-vectorized execution.")
             return '-D__SCALAR__'
 
-        result = subprocess.check_output('lscpu', shell=True)
-        result = result.decode('utf-8').strip().lower()
+        try:
+            result = subprocess.check_output('lscpu', shell=True)
+            result = result.decode('utf-8').strip().lower()
+        except Exception as e:
+            print(
+                f"{WARNING} {self.name} SIMD_WIDTH cannot be recognized due to {str(e)}!"
+            )
+            return '-D__SCALAR__'
+
         if 'genuineintel' in result:
             if 'avx512' in result:
                 return '-D__AVX512__'
@@ -432,7 +440,7 @@ class CUDAOpBuilder(OpBuilder):
                 cross_compile_archs = cross_compile_archs_env.replace(' ', ';')
             else:
                 if cross_compile_archs is None:
-                    cross_compile_archs = get_default_compute_capatabilities()
+                    cross_compile_archs = get_default_compute_capabilities()
             ccs = cross_compile_archs.split(';')
 
         args = []
