@@ -4,10 +4,208 @@ import importlib
 import warnings
 import torch
 import math
-from .trsrc import matmul
-from ..op_builder import SparseAttnBuilder
 
-triton = None
+import triton
+import triton.language as tl
+import triton._C.libtriton as libtriton
+
+
+@triton.jit
+def _kernel(A,
+            B,
+            C,
+            stride_za,
+            stride_ha,
+            stride_ma,
+            stride_ka,
+            stride_zb,
+            stride_hb,
+            stride_kb,
+            stride_nb,
+            stride_zc,
+            stride_hc,
+            stride_mc,
+            stride_nc,
+            DS0,
+            DS1,
+            SDD_K,
+            SDD_off_width,
+            lut,
+            locks,
+            nlocks,
+            **meta):
+    TM = meta['TM']
+    TN = meta['TN']
+    TK = meta['TK']
+    TZ = meta['TZ']
+    BLOCK = meta['BLOCK']
+    #------------#
+    #- Prologue -#
+    #------------#
+    pid0 = tl.program_id(0)
+    pid1 = tl.program_id(1)
+    pidz = tl.program_id(2)
+    if meta['SDD']:
+        pid1 = pid1 + SDD_off_width
+        blockidm = tl.arange(0, TM) // BLOCK
+        blockidn = tl.arange(0, TN) // BLOCK
+        offlutm = blockidm * (TN // BLOCK) * 4
+        offlutn = blockidn * 4
+        header = lut + pid1 * (TM // BLOCK) * (TN // BLOCK) * 4
+        z = tl.load(header + 0)
+        i = tl.load(header + 1 + offlutm)
+        j = tl.load(header + 2 + offlutn)
+        AS1 = SDD_K // TZ
+        lockid = tl.where(TZ > 1, 1, 0)
+        offka = pid0 * AS1
+        offkb = pid0 * AS1
+        offmc = 0
+        offnc = 0
+        offpa = 0
+        offpb = 0
+        maxid = TZ
+        offhc = 0
+        offha = z
+        offhb = z
+        ram = i * BLOCK + (tl.arange(0, TM) % BLOCK)
+        rbn = j * BLOCK + (tl.arange(0, TN) % BLOCK)
+    else:
+        header = lut + pid0 * 6
+        offset = tl.load(header + 0)
+        AS1 = tl.load(header + 1)
+        column = tl.load(header + 2)
+        depth = tl.load(header + 3)
+        lockid = tl.load(header + 4)
+        maxid = tl.load(header + 5)
+        pinc = lut + offset
+        offhc = depth
+        if meta['DSD']:
+            # output offset
+            offnc = pid1 * TN
+            offmc = column * TM
+            offpc = 0
+            # dense input offset
+            offnb = pid1 * TN
+            offkb = tl.load(pinc)
+            offkb = tl.multiple_of(offkb, 8)  # compiler hint
+            offpb = 0
+            # sparse input offset
+            offma = 0
+            offka = 0
+            offpa = tl.load(pinc + 1)
+            offpa = tl.multiple_of(offpa, 8)  # compiler hint
+            offpa = offpa * BLOCK * BLOCK
+            offha = 0
+            offhb = depth
+        else:
+            # output offset
+            offmc = pid1 * TM
+            offnc = column * TN
+            offpc = 0
+            # dense input offset
+            offma = pid1 * TM
+            offka = tl.load(pinc)
+            offka = tl.multiple_of(offka, 8)  # compiler hint
+            offpa = 0
+            # sparse input offset
+            offnb = 0
+            offkb = 0
+            offpb = tl.load(pinc + 1)
+            offpb = tl.multiple_of(offpb, 8)  # compiler hint
+            offpb = offpb * BLOCK * BLOCK
+            offha = depth
+            offhb = 0
+        ram = offma + tl.arange(0, TM)
+        rbn = offnb + tl.arange(0, TN)
+
+    # initialize a, b pointers
+    rka = offka + tl.arange(0, TK)
+    rkb = offkb + tl.arange(0, TK)
+    pa = A + pidz * stride_za + offha * stride_ha + offpa + ram[:, None] * stride_ma + rka[None, :] * stride_ka
+    pb = B + pidz * stride_zb + offhb * stride_hb + offpb + rbn[None, :] * stride_nb + rkb[:, None] * stride_kb
+    if meta['DDS']:
+        checkam = ram[:, None] < DS0
+    else:
+        checkam = AS1 > 0
+    if meta['DSD']:
+        checkbn = rbn[None, :] < DS0
+    else:
+        checkbn = AS1 > 0
+    a = tl.load(pa, mask=checkam, other=0.)
+    b = tl.load(pb, mask=checkbn, other=0.)
+
+    ## ---------------- ##
+    ##    Inner Loop    ##
+    ## ---------------- ##
+    acc = tl.zeros((TM, TN), dtype=tl.float32)
+    for k in range(AS1, 0, -TK):
+        acc += tl.dot(a, b)
+        if meta['SDD']:
+            inc_a = TK * stride_ka
+            inc_b = TK * stride_kb
+        else:
+            pinc += 2
+        if meta['DSD']:
+            inc_b = tl.load(pinc)
+            inc_a = tl.load(pinc + 1)
+            inc_b = tl.multiple_of(inc_b, 8)
+            inc_a = tl.multiple_of(inc_a, 8)
+            inc_b = inc_b * stride_kb
+        if meta['DDS']:
+            inc_a = tl.load(pinc)
+            inc_b = tl.load(pinc + 1)
+            inc_a = tl.multiple_of(inc_a, 8)
+            inc_b = tl.multiple_of(inc_b, 8)
+            inc_a = inc_a * stride_ka
+        pa += inc_a
+        pb += inc_b
+        # pre-fetch
+        checkak = k > TK
+        checkbk = k > TK
+        checka = checkam & checkak
+        checkb = checkbn & checkbk
+        a = tl.load(pa, mask=checka)
+        b = tl.load(pb, mask=checkb)
+    c = acc.to(C.dtype.element_ty)
+
+    if meta['SDD']:
+        checkc = True
+        rr_blockidm = tl.arange(0, TM) // BLOCK
+        rr_blockidn = tl.arange(0, TN) // BLOCK
+        rr_offlutm = rr_blockidm * (TN // BLOCK) * 4
+        rr_offlutn = rr_blockidn * 4
+        off_bkid = 3 + rr_offlutm[:, None] + rr_offlutn[None, :]
+        bkid = tl.load(header + off_bkid)
+        offpc = bkid * BLOCK * BLOCK
+        rcm = tl.arange(0, TM) % BLOCK
+        rcn = tl.arange(0, TN) % BLOCK
+    else:
+        rcm = offmc + tl.arange(0, TM)
+        rcn = offnc + tl.arange(0, TN)
+    if meta['DSD']:
+        checkc = rcn[None, :] < DS0
+    if meta['DDS']:
+        checkc = rcm[:, None] < DS0
+
+    pc = C + offpc + offhc * stride_hc + pidz * stride_zc + rcm[:, None] * stride_mc + rcn[None, :] * stride_nc
+    # write-back directly
+    if lockid == 0:
+        tl.store(pc, c, mask=checkc)
+    # accumulate partial results using spin-locks
+    else:
+        plock = locks + tl.program_id(2) * nlocks * tl.num_programs(1) + tl.program_id(
+            1) * nlocks + lockid - 1
+        pcount = plock + tl.num_programs(2) * tl.num_programs(1) * nlocks
+        while tl.atomic_cas(plock, 0, 1) == 1:
+            pass
+        count = tl.load(pcount)
+        if count == 0:
+            tl.store(pc, c, mask=checkc)
+        else:
+            d = tl.load(pc, mask=checkc)
+            tl.store(pc, d + c, mask=checkc)
+        tl.atomic_xchg(pcount, (count + 1) % maxid)
+        tl.atomic_xchg(plock, 0)
 
 
 ##############
@@ -26,9 +224,9 @@ class _sparse_matmul(torch.autograd.Function):
     # between `seg_size` elements
     @staticmethod
     def load_balance(sizes, block):
-        global triton
-        if triton is None:
-            triton = importlib.import_module('triton')
+        #global triton
+        #if triton is None:
+        #    triton = importlib.import_module('triton')
         # segment size
         # heuristics taken from OpenAI blocksparse code
         # https://github.com/openai/blocksparse/blob/master/blocksparse/matmul.py#L95
@@ -85,23 +283,22 @@ class _sparse_matmul(torch.autograd.Function):
     ##########################
     # SPARSE = DENSE x DENSE #
     ##########################
-    cpp_utils = None
-    sdd_segment = None
-
-    @staticmethod
-    def _load_utils():
-        if _sparse_matmul.cpp_utils is None:
-            _sparse_matmul.cpp_utils = SparseAttnBuilder().load()
-            _sparse_matmul.sdd_segment = _sparse_matmul.cpp_utils.sdd_segment
 
     @staticmethod
     def make_sdd_lut(layout, block, dtype, device):
-        _sparse_matmul._load_utils()
-        start_width = 64 // block
-        segmented = _sparse_matmul.sdd_segment(layout.type(torch.int32), start_width)
+        #_sparse_matmul._load_utils()
+        #start_width = 64 // block
+        #segmented = _sparse_matmul.sdd_segment(layout.type(torch.int32), start_width)
+        start_width = 128 // block
+        layout = layout.type(torch.int32)
+        segmented = libtriton.superblock(layout.data_ptr(),
+                                         layout.shape[0],
+                                         layout.shape[1],
+                                         layout.shape[2],
+                                         start_width)
         luts, widths, packs = [], [], []
         for size, nnz in segmented:
-            width = nnz.shape[0] // (size * size)
+            """ width = nnz.shape[0] // (size * size)
             h = nnz[:, 0]
             i = nnz[:, 1]
             j = nnz[:, 2]
@@ -109,6 +306,11 @@ class _sparse_matmul(torch.autograd.Function):
             lut = torch.stack((h, i, j, b), dim=1).view(-1).contiguous()
             luts.append(lut.type(torch.int32).to(device))
             widths.append(width)
+            packs.append(size) """
+            nnz = nnz.reshape(-1, 4)
+            width = nnz.shape[0] // (size * size)
+            luts.append(torch.from_numpy(nnz).type(torch.int32).to(device))
+            widths.append(width)
             packs.append(size)
         # create locks
         return luts, None, widths, packs
@@ -127,72 +329,55 @@ class _sparse_matmul(torch.autograd.Function):
                     packs,
                     bench,
                     time):
-        global triton
-        if triton is None:
-            triton = importlib.import_module('triton')
-
         if trans_c:
             a, b = b, a
             trans_a, trans_b = not trans_b, not trans_a
         AS0 = a.size(0)
-        AS1 = a.size(1)
-        AS2 = a.size(3 if trans_a else 2)
-        AS3 = a.size(2 if trans_a else 3)
-        BS0 = b.size(0)
-        BS1 = b.size(1)
-        BS2 = b.size(3 if trans_b else 2)
-        BS3 = b.size(2 if trans_b else 3)
+        # Shape check
+        a_dim = -2 if trans_a else -1
+        b_dim = -1 if trans_b else -2
+        a_inner, b_inner = a.shape[a_dim], b.shape[b_dim]
+        if a_inner != b_inner:
+            raise ValueError(
+                f"Size of tensor A along the {_dim_to_name(a_dim)} dim ({a_inner}) must match size "
+                f"of tensor B along the {_dim_to_name(b_dim)} dim ({b_inner})")
+        if a_inner % 16 != 0:
+            raise ValueError('Reduction size for SDD must be a multiple of 16')
+
+        batch_size = a.size(0)
+        a_outer = a.size(3 if trans_a else 2)
         dtype = a.dtype
-        is_16_multiple = AS3 % 16 == 0
-        is_32_multiple = AS3 % 32 == 0
-        is_64_multiple = AS3 % 64 == 0
+        is_16_multiple = a_inner % 16 == 0
+        is_32_multiple = a_inner % 32 == 0
+        is_64_multiple = a_inner % 64 == 0
         if not is_16_multiple:
             raise ValueError('Reduction size for SDD must be a multiple of 16')
+        device = a.device
         # create kernel
         total_width = sum([width * pack * pack for width, pack in zip(widths, packs)])
-        c = torch.empty((AS0, total_width, block, block), dtype=dtype, device=a.device)
+        c = torch.empty((batch_size,
+                         total_width,
+                         block,
+                         block),
+                        dtype=dtype,
+                        device=a.device)
         for lut, width, pack in zip(luts, widths, packs):
+            F32TK = [8, 16]
+            F16TK = [16]
+            F16TK += [32] if is_32_multiple else []
+            F16TK += [64] if is_64_multiple else []
+            TK = {torch.float32: F32TK, torch.float16: F16TK}[dtype]
             num_lock = 1
-            key = (block,
-                   a.dtype,
-                   b.dtype,
-                   trans_a,
-                   trans_b,
-                   trans_c,
-                   pack,
-                   is_32_multiple,
-                   is_64_multiple)
-            if key not in _sparse_matmul.sdd_cache:
-                F32TK = [8, 16]
-                F16TK = [16]
-                F16TK += [32] if is_32_multiple else []
-                F16TK += [64] if is_64_multiple else []
-                TK = {torch.float32: F32TK, torch.float16: F16TK}[dtype]
-                defines = {
-                    'TM': block * pack,
-                    'TN': block * pack,
-                    'TMN': block * block * pack * pack,
-                    'BLOCK': block,
-                    'TK': TK[0],
-                    'TYPE': dtype,
-                    'STRIDE_AM': '1' if trans_a else 'lda',
-                    'STRIDE_AK': 'lda' if trans_a else '1',
-                    'STRIDE_BN': 'ldb' if trans_b else '1',
-                    'STRIDE_BK': '1' if trans_b else 'ldb',
-                    'STRIDE_CM': 'ldc',
-                    'STRIDE_CN': '1',
-                    'SDD': True,
-                    'TZ': 1,
-                    'NAME': 'sdd_kernel'
-                }
-                _sparse_matmul.sdd_cache[key] = triton.kernel(
-                    matmul,
-                    defines=defines,
-                    device=torch.device('cuda'),
-                    num_warps=4)
-                #_sparse_matmul.sdd_cache[key] = triton.kernel(src, defines=defines, num_warps=[1, 2, 4])
-
-            kernel = _sparse_matmul.sdd_cache[key]
+            meta = {
+                'TM': block * pack,
+                'TN': block * pack,
+                'BLOCK': block,
+                'TK': TK[0],
+                'TZ': 1,
+                'SDD': True,
+                'DSD': False,
+                'DDS': False
+            }
             # create output
             locks = _sparse_matmul.get_locks(2 * width * AS0 * num_lock, a.device)
             # maximum grid size is 65535
@@ -201,32 +386,36 @@ class _sparse_matmul(torch.autograd.Function):
             max_width = 49152
             total = 0 if bench else None
             for off_width in range(0, width, max_width):
-                current = kernel(
-                    a.data_ptr(),
-                    b.data_ptr(),
-                    c.data_ptr(),
-                    a.stride(2),
-                    b.stride(2),
-                    block,
-                    a.stride(0),
-                    b.stride(0),
-                    c.stride(0),
-                    a.stride(1),
-                    b.stride(1),
-                    c.stride(0),
-                    AS2,
-                    AS2,
-                    AS3,
-                    off_width,
-                    lut.data_ptr(),
-                    locks.data_ptr(),
-                    num_lock,
-                    grid=lambda opt: [opt.TZ,
-                                      min(max_width,
-                                          width - off_width),
-                                      AS0])
-                total = total + current if bench else None
-            time[0] = total
+                grid = lambda meta: [
+                    meta['TZ'],
+                    min(max_width,
+                        width - off_width),
+                    batch_size
+                ]
+                _kernel[grid](a,
+                              b,
+                              c,
+                              a.stride(0),
+                              a.stride(1),
+                              a.stride(3 if trans_a else 2),
+                              a.stride(2 if trans_a else 3),
+                              b.stride(0),
+                              b.stride(1),
+                              b.stride(3 if trans_b else 2),
+                              b.stride(2 if trans_b else 3),
+                              c.stride(0),
+                              c.stride(0),
+                              c.stride(2),
+                              c.stride(3),
+                              a_outer,
+                              a_outer,
+                              a_inner,
+                              off_width,
+                              lut,
+                              locks,
+                              num_lock,
+                              num_warps=4,
+                              **meta)
         # save for backward pass
         return c
 
@@ -358,31 +547,16 @@ class _sparse_matmul(torch.autograd.Function):
         BS2 = block * spdims[1 if trans_b else 2]
         dtype = a.dtype
         # kernel
-        key = (block, a.dtype, b.dtype, trans_a, trans_b, trans_c)
-        if key not in _sparse_matmul.dds_cache:
-            #TM = (64, 128) if dtype == torch.float32 else (64, 128, 256)
-            TK = 8 if dtype == torch.float32 else 16
-            defines = {
-                'TM': 128,
-                'TN': block,
-                'TK': TK,
-                'BLOCK': block,
-                'TYPE': dtype,
-                'STRIDE_AM': 1 if trans_a else 'lda',
-                'STRIDE_AK': 'lda' if trans_a else 1,
-                'STRIDE_BN': block if trans_b else 1,
-                'STRIDE_BK': 1 if trans_b else block,
-                'STRIDE_CM': '1' if trans_c else 'ldc',
-                'STRIDE_CN': 'ldc' if trans_c else '1',
-                'NAME': 'dds_kernel',
-                'DDS': True
-            }
-            _sparse_matmul.dds_cache[key] = triton.kernel(matmul,
-                                                          defines=defines,
-                                                          device=torch.device('cuda'),
-                                                          num_warps=4)
-            #_sparse_matmul.dds_cache[key] = triton.kernel(src, defines=defines, num_warps=[4])
-        kernel = _sparse_matmul.dds_cache[key]
+        meta = {
+            'TN': block,
+            'TM': 128,
+            'TK': 16,
+            'BLOCK': block,
+            'TZ': 1,
+            'SDD': False,
+            'DSD': False,
+            'DDS': True
+        }
         # output
         CS0 = AS0
         CS1 = AS1
@@ -390,29 +564,31 @@ class _sparse_matmul(torch.autograd.Function):
         CS3 = AS2 if trans_c else BS2
         locks = _sparse_matmul.get_locks(2 * AS0 * AS2 // 32 * num_locks, a.device)
         c = torch.empty((CS0, CS1, CS2, CS3), dtype=dtype, device=a.device)
-        time[0] = kernel(a.data_ptr(),
-                         b.data_ptr(),
-                         c.data_ptr(),
-                         a.stride(2),
-                         block,
-                         c.stride(2),
-                         a.stride(0),
-                         b.stride(0),
-                         c.stride(0),
-                         a.stride(1),
-                         b.stride(1),
-                         c.stride(1),
-                         AS2,
-                         BS2,
-                         0,
-                         0,
-                         lut.data_ptr(),
-                         locks.data_ptr(),
-                         num_locks,
-                         grid=lambda opt: [width,
-                                           triton.cdiv(AS2,
-                                                       opt.TM),
-                                           AS0])
+        grid = lambda meta: [width, triton.cdiv(AS2, meta['TM']), AS0]
+        _kernel[grid](a,
+                      b,
+                      c,
+                      a.stride(0),
+                      a.stride(1),
+                      a.stride(3 if trans_a else 2),
+                      a.stride(2 if trans_a else 3),
+                      b.stride(0),
+                      b.stride(1),
+                      b.stride(3 if trans_b else 2),
+                      b.stride(2 if trans_b else 3),
+                      c.stride(0),
+                      c.stride(1),
+                      c.stride(3 if trans_c else 2),
+                      c.stride(2 if trans_c else 3),
+                      AS2,
+                      BS2,
+                      0,
+                      0,
+                      lut,
+                      locks,
+                      num_locks,
+                      num_warps=4,
+                      **meta)
         return c
 
     @staticmethod
@@ -443,31 +619,17 @@ class _sparse_matmul(torch.autograd.Function):
         BS3 = b.size(2 if trans_b else 3)
         dtype = a.dtype
         # kernel
-        key = (block, a.dtype, b.dtype, trans_a, trans_b, trans_c)
-        if key not in _sparse_matmul.dsd_cache:
-            #TN = (64, 128) if dtype == torch.float32 else (64, 128, 256)
-            TK = 8 if dtype == torch.float32 else 16
-            defines = {
-                'TM': block,
-                'TN': 128,
-                'TK': TK,
-                'BLOCK': block,
-                'TYPE': dtype,
-                'STRIDE_AM': 1 if trans_a else block,
-                'STRIDE_AK': block if trans_a else 1,
-                'STRIDE_BN': 'ldb' if trans_b else '1',
-                'STRIDE_BK': '1' if trans_b else 'ldb',
-                'STRIDE_CM': '1' if trans_c else 'ldc',
-                'STRIDE_CN': 'ldc' if trans_c else '1',
-                'NAME': 'dsd_kernel',
-                'DSD': True
-            }
-            _sparse_matmul.dsd_cache[key] = triton.kernel(matmul,
-                                                          defines=defines,
-                                                          device=torch.device('cuda'),
-                                                          num_warps=4)
-            #_sparse_matmul.dsd_cache[key] = triton.kernel(src, defines=defines, num_warps=[4])
-        kernel = _sparse_matmul.dsd_cache[key]
+
+        meta = {
+            'TM': block,
+            'TN': 128,
+            'TK': 16,
+            'BLOCK': block,
+            'TZ': 1,
+            'SDD': False,
+            'DSD': True,
+            'DDS': False
+        }
         # output
         CS0 = BS0
         CS1 = BS1
@@ -475,29 +637,31 @@ class _sparse_matmul(torch.autograd.Function):
         CS3 = AS1 if trans_c else BS3
         locks = _sparse_matmul.get_locks(2 * BS0 * BS3 // 32 * num_locks, a.device)
         c = torch.empty((CS0, CS1, CS2, CS3), dtype=dtype, device=a.device)
-        time[0] = kernel(a.data_ptr(),
-                         b.data_ptr(),
-                         c.data_ptr(),
-                         block,
-                         b.stride(2),
-                         c.stride(2),
-                         a.stride(0),
-                         b.stride(0),
-                         c.stride(0),
-                         a.stride(1),
-                         b.stride(1),
-                         c.stride(1),
-                         BS3,
-                         AS1,
-                         0,
-                         0,
-                         lut.data_ptr(),
-                         locks.data_ptr(),
-                         num_locks,
-                         grid=lambda opt: [width,
-                                           triton.cdiv(BS3,
-                                                       opt.TN),
-                                           BS0])
+        grid = lambda meta: [width, triton.cdiv(BS3, meta['TN']), BS0]
+        _kernel[grid](a,
+                      b,
+                      c,
+                      a.stride(0),
+                      a.stride(1),
+                      a.stride(3 if trans_a else 2),
+                      a.stride(2 if trans_a else 3),
+                      b.stride(0),
+                      b.stride(1),
+                      b.stride(3 if trans_b else 2),
+                      b.stride(2 if trans_b else 3),
+                      c.stride(0),
+                      c.stride(1),
+                      c.stride(2),
+                      c.stride(3),
+                      BS3,
+                      AS1,
+                      0,
+                      0,
+                      lut,
+                      locks,
+                      num_locks,
+                      num_warps=4,
+                      **meta)
         return c
 
     fn = {
@@ -628,7 +792,7 @@ class MatMul:
             return self.lut_cache[key]
         # C look-up table
         layout, block = self.layout, self.block
-        step = 8 if dtype == torch.float32 else 16
+        step = 16
         if self.mode == 'sdd':
             c_lut, c_num_locks, c_width, c_packs = _sparse_matmul.make_sdd_lut(layout, block, dtype, device)
         elif self.mode == 'dsd':
@@ -674,9 +838,31 @@ class MatMul:
         self.trans_a = trans_a
         self.trans_b = trans_b
         self.mode = mode
-        self.spdims = layout.shape
         self.block = block
         self.layout = layout
+        layout_dim = layout.ndim
+        assert layout_dim in (2, 3), "Layout should be a 2 or 3 dimensional tensor of 0s and 1s"
+        if not mode == 'sdd':
+            # Dims to be reduced on the 'inside' of the matmul, either -1 or -2
+            trans_dense, trans_sparse, sparse_inner = (trans_b, trans_a, -1) if mode == 'dsd' else (trans_a, trans_b, -2)
+            self.dense_inner_dim = -(
+                (sparse_inner % 2) + 1) if not trans_dense else sparse_inner
+            sparse_inner = sparse_inner if not trans_sparse else -(
+                (sparse_inner % 2) + 1)
+
+            # Inner dim of the dense input should be equal to the inner dim of the sparse input
+            self.dense_inner_size = layout.shape[sparse_inner] * block
+            # Expected shape for sparse inputs
+            self.sparse_shape = (layout.sum().item(), block, block)
+
+        # Support using the same layout across attention heads etc.
+        if layout_dim == 2:
+            layout = layout.unsqueeze(0)
+
+        layout = layout.long(
+        )  # Above code assumes the layout tensor is an integral type
+
+        self.spdims = layout.shape
         # timings
         self.bench = bench
         self.time_c = None
@@ -713,10 +899,15 @@ class MatMul:
         time_c = [None]
         time_da = [None]
         time_db = [None]
+
+        original_dims = max(a.ndim, b.ndim)
+        a, b = self._validate_inputs(a, b)
+
         # pad shapes with ones
         a = MatMul._pad_shape(a, self.mode == 'dsd')
         b = MatMul._pad_shape(b, self.mode == 'dds')
         # execute
+
         c = _sparse_matmul.apply(a,
                                  b,
                                  self.trans_a,
@@ -743,7 +934,62 @@ class MatMul:
                                  db_packs,
                                  self.bench,
                                  time_db)
+
+        # This removes any leading singleton dimensions we may have added to the tensor that weren't in the input
+        dims_to_trim = c.ndim - original_dims
+        for _ in range(dims_to_trim):
+            c = c.squeeze(0)
+
         self.time_c = time_c[0]
         self.time_da = time_da[0]
         self.time_db = time_db[0]
         return c
+
+    def _validate_inputs(self, a, b):
+        if a.device != b.device:
+            raise ValueError(
+                f"Inputs must be on the same device; got {a.device} for tensor A "
+                f"and {b.device} for tensor B")
+        if not a.is_cuda:
+            raise ValueError("Only GPU devices are supported for now")
+
+        # When autocast is enabled, torch.matmul autocasts to float16, so we do the same here
+        if torch.is_autocast_enabled():
+            a, b = a.half(), b.half()
+        elif a.dtype != b.dtype:
+            raise ValueError(
+                f"Inputs must be the same dtype; got {a.dtype} for A and {b.dtype} for B"
+            )
+
+        mode, trans_a, trans_b = self.mode, self.trans_a, self.trans_b
+        if mode != 'sdd':
+            # One input is sparse
+            dense, dense_name, sparse, sparse_name = (a, 'A', b, 'B') if mode == 'dds' else (b, 'B', a, 'A')
+            dense_inner = dense.shape[self.dense_inner_dim]
+            if dense_inner != self.dense_inner_size:
+                raise ValueError(
+                    f"Expected tensor {dense_name} to have size {self.dense_inner_size} at dim "
+                    f"{self.dense_inner_dim % dense.ndim}, got {dense_inner}.")
+
+            if sparse.shape[-len(self.sparse_shape):] != self.sparse_shape:
+                raise ValueError(
+                    f"Expected tensor with trailing dimensions of shape {self.sparse_shape} for argument "
+                    f"{sparse_name}, got {sparse.shape}")
+
+        def add_extra_dims(x):
+            # Add extra leading singleton dimensions if needed
+            dims_needed = 4 - x.ndim
+            if dims_needed > 0:
+                singletons = [1] * dims_needed
+                x = x.view(*singletons, *x.shape)
+            elif dims_needed < 0:
+                raise ValueError(
+                    "Tensors with more than 4 dimensions are not currently supported")
+
+            return x
+
+        # Pad shapes with leading singleton dimensions
+        a = add_extra_dims(a)
+        b = add_extra_dims(b)
+
+        return a, b
